%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Summer School StrongMAR - 2016
% Array Data Processing
% Paulo Santos 
% 29th June, 2016
% Frequency domain beamformer 3D
% Different array configurations
% Introducing VSA
% pjsantos@ualg.pt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%close all
clear all

%sampling frequency
fs=5000;
NFFT=2048;
N=256;

%planewave simulation

f=750;  %signal frequency
%%%DOA of received signal
azi=40%input('azimuth angle (plano xy)(degrees)? ')
tetha=(azi)*pi/180; 
elev=20%input('elevation angle (with z axis)(degrees)? ')
phi=(elev)*pi/180; 

L=9;%input('n of sensors? ')
d=1;%input('sensors spacing (m)? ')

u=(-1).*[cos(tetha)*cos(phi) sin(tetha)*cos(phi) sin(phi)]; %direction cosines 
v=u; 
w=2*pi*f;
c=1500; %sound speed m/s

%pressure
P=1;
ro=1;
vp=-1;%%P/(ro*c);

%%Array configuration
l=1:L; %(9 sensors)
r(l)=[(l-1)*d];
%rs=[zeros(2,L);-r]; %location of sensors (z axis)
%rs=[r;zeros(2,L)]; %location of sensors (x axis)
%rs=[zeros(1,L);r;zeros(1,L)]; %location of sensors (y axis)

%%%%%%%planar xy (9 sensors)
% rs=[0  1  -1  0  0  2  0 -2  0
%     0  0   0  1 -1  0  2  0 -2
%     0  0   0  0  0  0  0  0  0];

%%%%%%%planar xz (9 sensors)
% rs=[0 1 -1  0  0  2  0 -2  0
%     0 0 0 0 0 0 0 0 0
%     0 0   0  1 -1  0  2  0 -2];


k=w/c*u;
tau=k*rs; %delays

t=0:1/fs:NFFT/fs;

for n=1:NFFT
    %received signal with delay
   Vx(n,l)=v(1)*exp(i*(w*t(n)-tau(l)));
   Vy(n,l)=v(2)*exp(i*(w*t(n)-tau(l)));
   Vz(n,l)=v(3)*exp(i*(w*t(n)-tau(l)));
   Vp(n,l)=vp*exp(i*(w*t(n)-tau(l)));
end 

%frequency domain signal
VX=fft(Vx); 
VY=fft(Vy);
VZ=fft(Vz);
VP=fft(Vp);

%figure
%plot(0:fs/NFFT:fs-1,abs(VP))
%axis tight

% steer of signal frequency
%(indf-1)*fs/NFFT=frequency
[maxfx indfx]=max(VX(:,1));
[maxfy indfy]=max(VY(:,1));
[maxfz indfz]=max(VZ(:,1));
[maxfp indfp]=max(VP(:,1));

VXrs=VX(indfx,:);
VYrs=VY(indfy,:);
VZrs=VZ(indfz,:); 
VPrs=VP(indfp,:);


for a_s=1:361
    for e_s=1:181
        azi(a_s)=a_s-181;
        elev(e_s)=91-e_s;
        tetha_s=(a_s-181)*pi/180;
        phi_s=(91-e_s)*pi/180;
	    u_s=(-1).*[cos(tetha_s)*cos(phi_s) sin(tetha_s)*cos(phi_s) sin(phi_s)];  
        k_s=w/c*u_s;
        tau_s=k_s*rs;
        wn=[1 cos(tetha_s)*cos(phi_s) sin(tetha_s)*cos(phi_s) sin(phi_s)];
        
	    Xss=exp(-j*tau_s); %simul signal
        
        Vrs=[VPrs;VXrs;VYrs;VZrs];
        Bp(a_s,e_s)=(abs(VPrs*Xss'))^2; %only omnidireccional
        B(a_s,e_s)=(abs(wn*Vrs*Xss'))^2; %VSA
    end
end
[bmaxe, binde]=(max(max(B)));
[bmaxa binda]=(max(max(B')));
Bnorm=B./bmaxe;
Bpnorm=Bp./max(max(Bp));

figure
pcolor(azi,elev,Bpnorm')
shading interp
xh=xlabel('\theta ( \circ)','fontsize',14);
yh=ylabel('\phi ( \circ)','fontsize',14);
axis([-180 180 -90 90]);
set(gca,'fontsize',14)
title(['DOA (only p) - f = ', num2str(f),' Hz'],'fontsize',14);
colorbar
set(colorbar,'fontsize',12)
pause

figure
pcolor(azi,elev,Bnorm')
shading interp
xh=xlabel('\theta ( \circ)','fontsize',14);
yh=ylabel('\phi ( \circ)','fontsize',14);
axis([-180 180 -90 90]);
set(gca,'fontsize',14)
title(['DOA (VSA) - f = ', num2str(f),' Hz'],'fontsize',14);
colorbar
set(colorbar,'fontsize',12)

pause
figure
%sphere3d(Bp',-pi,pi,-pi/2,pi/2,1,1,'surf');
[x,y,z]=sphere3d(Bp',-pi,pi,-pi/2,pi/2,1,1,'surf');
shading interp
s=get(gca,'CameraPosition');
p=get(gca,'CameraTarget');
x=get(gca,'Xlim');
y=get(gca,'Ylim');
z=get(gca,'Zlim');

line('xdata',x);
line('ydata',y);
line('zdata',z);

text(x(2),0,0,'x','fontsize',14,'fontweight','light','fontangle','italic');
text(0,y(2),0,'y','fontsize',14,'fontweight','light','fontangle','italic');
text(0,0,z(2),'z','fontsize',14,'fontweight','light','fontangle','italic');
colorbar
set(colorbar,'fontsize',12)

figure
%sphere3d(B',-pi,pi,-pi/2,pi/2,1,1,'surf');
[x,y,z]=sphere3d(B',-pi,pi,-pi/2,pi/2,1,1,'surf');
shading interp
s=get(gca,'CameraPosition');
p=get(gca,'CameraTarget');
x=get(gca,'Xlim');
y=get(gca,'Ylim');
z=get(gca,'Zlim');

line('xdata',x);
line('ydata',y);
line('zdata',z);

text(x(2),0,0,'x','fontsize',14,'fontweight','light','fontangle','italic');
text(0,y(2),0,'y','fontsize',14,'fontweight','light','fontangle','italic');
text(0,0,z(2),'z','fontsize',14,'fontweight','light','fontangle','italic');
colorbar
set(colorbar,'fontsize',12)

